#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

#region Known Limitations
/*
The template has the following known limitations:

- Usability limitations
On page edit is disabled.
Added comment only shows up in list after reload.

- Multi language limitations
Due to the nature of a blog, the template is not designed and tested with multi languge considerations in mind.

*/
#endregion

using System;
using System.Collections.Generic;

using EPiServer;
using EPiServer.Core;
using EPiServer.DataAbstraction;
using EPiServer.Security;
using EPiServer.Personalization;
using System.Text.RegularExpressions;

namespace EPiServer.Templates.Demo.Blog.Pages
{
    /// <summary>
    /// Template used for blog item page type.
    /// </summary>
    public partial class Item : BlogPageBase
    {
        private readonly string _removeInvalidRegEx = @"(</?)(i|p|b|strong|em)[^>]*(>)|<[^>]*>";
        private readonly string _restoreValidTagsRegEx = @"&lt;(/?)(i|p|b|strong|em)&gt;";
        private List<PageData> _tags;

        /// <summary>
        /// Checks if comments and/or captcha should be displayed.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"></see> object that contains the event data.</param>
        protected override void OnPreRender(EventArgs e)
        {
            base.OnPreRender(e);

            bool allowAnonymousComments = BlogStart[BlogUtil.DisableAnonymousCommentsPropertyName] == null;
            bool isLoggedIn = Context.User.Identity.IsAuthenticated;

            if (isLoggedIn)
            {
                CommentArea.Visible = true;
                Captcha.Visible = false;
                CommentName.Text = EPiServerProfile.Current.DisplayName;
            }
            else if (allowAnonymousComments)
            {
                CommentArea.Visible = true;
                Captcha.Visible = true;
            }
            else
            {
                CommentArea.Visible = false;
                Captcha.Visible = false;
            }

            if (!IsPostBack)
            {
                DataBind();
            }
        }

        /// <summary>
        /// The source for RSS/Atom feeds, which in this case is the comment pages.
        /// </summary>
        /// <value>All children to this blog item.</value>
        protected override PageDataCollection FeedSource
        {
            get
            {
                return DataFactory.Instance.GetChildren(CurrentPage.PageLink);
            }
        }

        /// <summary>
        /// The tags for the blog item
        /// </summary>
        protected List<PageData> Tags
        {
            get
            {
                if (_tags == null)
                {
                    _tags = new List<PageData>();
                    CategoryList cl = (CategoryList)CurrentPage["PageCategory"];

                    if (cl == null)
                    {
                        return _tags;
                    }

                    Category categoryTag;
                    PageDataCollection pdc = DataFactory.Instance.GetChildren((PageReference)BlogStart[BlogUtil.TagContainerPropertyName]);

                    foreach (int c in cl)
                    {
                        categoryTag = Category.Find(c);

                        foreach (PageData pd in pdc)
                        {
                            if (pd.PageName == categoryTag.Name)
                            {
                                _tags.Add(pd);
                            }
                        }
                    }
                }
                return _tags;
            }
        }

        /// <summary>
        /// Creates a new comment page.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void PostComment_Click(object sender, System.EventArgs e)
        {
            // Make sure that the page, and particurlarly the captcha, is valid. 
            if (IsValid) 
            {
                PageData newComment = DataFactory.Instance.GetDefaultPageData(CurrentPage.PageLink, BlogUtil.CommentPageTypeName, AccessLevel.NoAccess);

                // comment is used by a PropertyXhtmlString, and XhtmlString are built to work with an editor. Hence, they expect encoded text. 
                // Therefor we must html encode the content.
                string comment = Server.HtmlEncode(Regex.Replace(CommentText.Text, _removeInvalidRegEx, "$1$2$3", RegexOptions.IgnoreCase));

                // name will be saved in a PropertyString, and should not be encoded. PropertyString encodeds itself when it's rendered.
                string name = Regex.Replace(CommentName.Text, _removeInvalidRegEx, "$1$2$3", RegexOptions.IgnoreCase);
                
                comment = Regex.Replace(comment, _restoreValidTagsRegEx, "<$1$2>", RegexOptions.IgnoreCase);
                name = Regex.Replace(name, _restoreValidTagsRegEx, "<$1$2>", RegexOptions.IgnoreCase);

                newComment.PageName = DateTime.Now.ToString();
                newComment["MainBody"] = comment;
                newComment["Writer"] = name;
                newComment.VisibleInMenu = false;

                // To get the new comment into the list. If we don't do this the
                // StartPublish value will be greater than the one used for
                // the publishing-filter.
                newComment.StartPublish = DateTime.Now.AddMinutes(-1);

                DataFactory.Instance.Save(newComment, EPiServer.DataAccess.SaveAction.Publish, AccessLevel.NoAccess);

                CommentText.Text = String.Empty;
            }
        }
    }
}
