#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using System.Data;
using System.Configuration;
using System.Collections;
using System.IO;
using System.Web;
using System.Web.Hosting;
using System.Web.Security;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Web.UI.WebControls.WebParts;
using System.Web.UI.HtmlControls;

using EPiServer.BaseLibrary.IO;
using EPiServer.Demo.FileManager;
using EPiServer.Demo.FileManager.WebControls;
using EPiServer.Security;
using EPiServer.Web.Hosting;
using EPiServer.Web.WebControls;

namespace EPiServer.Templates.Demo.FileManager
{
    /// <summary>
    /// Uploads a new file or replaces an existing in the <see cref="FileManagerControl.CurrentVirtualDirectory"/>
    /// </summary>
    public partial class AddFile : ContentUserControlBase
    {

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);
            if (!IsPostBackView)
            {
                if (!FileManager.CurrentVirtualDirectory.QueryDistinctAccess(AccessLevel.Create | AccessLevel.Edit | AccessLevel.Delete))
                {
                    UploadButton.Enabled = false;
                    Page.Validators.Add(new StaticValidator(Translate("/filemanager/errormessage/insufficientaccess")));
                    return;
                }
                ResetUploadForm();
            }
        }

        /// <summary>
        /// Handles file upload when the UploadFile button is clicked.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void UploadFile_Click(object sender, EventArgs e)
        {
            if (FileUpload.HasFile)
            {
                //// Check if there already exists a file with the same name 
                VirtualFile existingFile = FileSystemUtility.GetFile(FileManager.CurrentVirtualDirectory, FileUpload.FileName);

                if (existingFile == null)
                {
                    // No file found. Let's create a new file.
                    UnifiedFile targetFile = FileManager.CurrentVirtualDirectory.CreateFile(FileUpload.FileName);
                    SaveStreamToFile(FileUpload.PostedFile.InputStream, targetFile, CheckInComment.Text);
                    ResetUploadForm();
                    FileManager.RaiseCommand(this, new FileManagerCommandEventArgs(FileManagerCommandName.ChildCollectionChanged, FileManager.CurrentVirtualPath));
                    DataBindChildren();
                    //TODO: Notify filemanager of newly added file RaiseCommand(ChildCollectionChanged)
                }
                else 
                {
                    // A file with the same name already exists.
                    IVersioningFile versioningFile = existingFile as IVersioningFile;
                    if (versioningFile.IsCheckedOut && !String.Equals(versioningFile.CheckedOutBy, Page.User.Identity.Name))
                    {
                        // File is versioned and checked out to someone else
                        Page.Validators.Add(new StaticValidator(Translate("/filemanager/errormessage/checkedout")));
                        return;
                    }

                    // File already exists; create a temp file for upload and alert the user.
                    UnifiedFile targetFile = FileSystemUtility.CreateTempFile(FileManager.CurrentVirtualDirectory);
                    SaveStreamToFile(FileUpload.PostedFile.InputStream, targetFile, null);

                    TemporaryFileName = targetFile.Name;
                    DestinationFileName = FileUpload.FileName;

                    ReplaceFileButton.Visible = true;
                    Page.Validators.Add(new StaticValidator(String.Format(Translate("/filemanager/errormessage/replacefile"), DestinationFileName)));

                    FileUploadPanel.Visible = false;
                    UploadButton.Visible = false;
                    CommentsPanel.Visible = false;
                }
            }
        }

        /// <summary>
        /// Handles the Click event of the ReplaceFile control.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void ReplaceFile_Click(object sender, EventArgs e)
        {
            UnifiedFile temporaryFile = FileSystemUtility.GetFile(FileManager.CurrentVirtualDirectory, TemporaryFileName) as UnifiedFile;
            UnifiedFile destinationFile = FileSystemUtility.GetFile(FileManager.CurrentVirtualDirectory, DestinationFileName) as UnifiedFile;

            if (temporaryFile != null && destinationFile != null)
            {
                using (Stream sourceStream = temporaryFile.Open())
                {
                    SaveStreamToFile(sourceStream, destinationFile, CheckInComment.Text);
                }
                temporaryFile.Delete();
                ResetUploadForm();
                FileManager.RaiseCommand(this, new FileManagerCommandEventArgs(FileManagerCommandName.ChildCollectionChanged, FileManager.CurrentVirtualPath));
                DataBindChildren();
            }
        }

        /// <summary>
        /// Handles the Click event of the Cancel button to cancel the current action.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CancelButton_Click(object sender, EventArgs e)
        {
            if (!String.IsNullOrEmpty(TemporaryFileName))
            {
                // File was uploaded to a temporary location, but the replace was aborted.
                UnifiedFile tempFile = FileSystemUtility.GetFile(FileManager.CurrentVirtualDirectory, TemporaryFileName) as UnifiedFile;
                if (tempFile != null)
                {
                    tempFile.Delete();
                }
                ResetUploadForm();
            }
            else 
            {
                FileManager.LoadDefaultView(this);
            }
        }

        /// <summary>
        /// Saves the source stream to a unified file and adds a comment if the file implrments IVersioningFile.
        /// </summary>
        /// <param name="sourceStream">The source stream.</param>
        /// <param name="targetFile">The target file.</param>
        /// <param name="comment">A comment to add when checking in a versioning file.</param>
        private void SaveStreamToFile(Stream sourceStream, UnifiedFile targetFile, string comment)
        {
            // It may be a versioning file
            IVersioningFile versioningFile = targetFile as IVersioningFile;

            if (versioningFile != null)
            {
                // Make sure versioning files are checked out, otherwise we'll get an automatic check-in without comments.
                if (!versioningFile.IsCheckedOut)
                {
                    versioningFile.CheckOut();
                }
            }

            // Copy the source stream to the target file stream.
            using (Stream writeStream = targetFile.Open(FileMode.Create, FileAccess.Write))
            {
                StreamConsumer.CopyToEnd(sourceStream, writeStream);
            }

            // If versioning, then check in with the supplied comment.
            if (versioningFile != null)
            {
                versioningFile.CheckIn(comment ?? String.Empty);
            }

        }


        /// <summary>
        /// Resets the upload form to the initial state.
        /// </summary>
        private void ResetUploadForm()
        {
            UploadButton.Visible = UploadButton.Enabled = true;
            FileUploadPanel.Visible = true;
            
            // Show comments panel only for versioning files
            CommentsPanel.Visible = FileManager.CurrentVirtualDirectory.IsVersioningSupported;
            CheckInComment.Text = String.Empty;
            
            ReplaceFileButton.Visible = false;
            DestinationFileName = TemporaryFileName = null;
        }


        /// <summary>
        /// Gets or sets the name of the temporary file used when replacing an existing file.
        /// </summary>
        /// <value>The name of the temporary file.</value>
        private string TemporaryFileName 
        {
            get { return ViewState["TempFileName"] as string; }
            set { ViewState["TempFileName"] = value; }
        }


        /// <summary>
        /// Gets or sets the name of the destination file when replacing an existing file.
        /// </summary>
        /// <value>The name of the temporary file.</value>
        private string DestinationFileName 
        {
            get { return ViewState["DestinationFileName"] as string; }
            set { ViewState["DestinationFileName"] = value; }
        }

    }
}