#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using System.Web.Security;
using System.Web.UI.WebControls;

using EPiServer;
using EPiServer.Personalization;

namespace EPiServer.Templates.Demo.Units.Placeable
{
    public partial class PersonalSettings : UserControlBase
    {
        private void Page_Load(object sender, System.EventArgs e)
        {
            if (!IsPostBack)
            {
                HideFields();
            }
            InitializeFields();
        }

        /// <summary>
        /// If the user isn't logged on we hide the panel containing the form.
        /// If the user is logged on the data from the user is loaded to the form.
        /// </summary>
        protected virtual void InitializeFields()
        {
            if (!Page.User.Identity.IsAuthenticated)
            {
                CreateEditUser.Visible = false;
                return;
            }
            if (!IsPostBack)
            {
                LoadUserData();
            }
        }

        /// <summary>
        /// Checks that the user is logged on. If he is, we save the user. If not we add a message and hides the panel containing the form.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected virtual void ApplyButton_Click(object sender, EventArgs e)
        {
            if (!Page.User.Identity.IsAuthenticated)
            {
                ErrorOccured(Translate("/templates/personalsettings/expiredidentity"));
                CreateEditUser.Visible = false;
                return;
            }
            SaveUser();
        }

        /// <summary>
        /// Saves the current profile.
        /// </summary>
        /// <returns></returns>
        protected virtual bool SaveUser()
        {
            if (!Page.IsValid)
            {
                return false;
            }

            SetPostedData();
            try
            {
                EPiServerProfile.Current.Save();
                SaveFailed.Visible = false;
                SaveSucceded.Visible = true;
                CreateEditUser.Visible = false;
            }
            catch (Exception error)
            {
                ErrorOccured(error.Message);
                return false;
            }
            return true;
        }

        /// <summary>
        /// Displays any error that occurs and hides the panel containing the form.
        /// </summary>
        /// <param name="errorMessage">The error message.</param>
        protected void ErrorOccured(string errorMessage)
        {
            SaveFailed.Visible = true;
            SaveSucceded.Visible = false;
            ErrorMessage.Text = errorMessage;
        }

        /// <summary>
        /// Loads the user data from the current profile to the form.
        /// </summary>
        private void LoadUserData()
        {
            if (EPiServerProfile.Enabled)
            {
                // Get preset properties
                EPiServerProfile user = EPiServerProfile.Current;
                Company.Text = user.Company;
                Email.Text = user.Email;
                Firstname.Text = user.FirstName;
                if (!String.IsNullOrEmpty(user.Language))
                {
                    Language.SelectedLanguage = EPiServer.DataAbstraction.LanguageBranch.Load(user.Language);
                }
                Lastname.Text = user.LastName;
                Title.Text = user.Title;
                Country.Text = user.Country;
            }
        }

        /// <summary>
        /// Adds the data from the form to the current profile.
        /// </summary>
        protected void SetPostedData()
        {
            EPiServerProfile user = EPiServerProfile.Current;

            user.Company = StripAnyTags(Company.Text);
            user.Email = StripAnyTags(Email.Text);
            user.FirstName = StripAnyTags(Firstname.Text);
            user.Language = StripAnyTags(Language.SelectedLanguage == null ? user.Language : Language.SelectedLanguage.LanguageID);
            user.LastName = StripAnyTags(Lastname.Text);
            user.Title = StripAnyTags(Title.Text);
            user.Country = StripAnyTags(Country.Text);
        }

        /// <summary>
        /// Strips any tags by removing any great then or less then caracters from the string.
        /// </summary>
        /// <param name="s">A string that should be stripped from tags.</param>
        /// <returns>A string without tags</returns>
        protected string StripAnyTags(string s)
        {
            return s == null ? null : s.Replace("<", "").Replace(">", "");
        }

        /// <summary>
        /// Hides any fields that are set up to be disabled in edit mode.
        /// </summary>
        protected void HideFields()
        {
            CountryRow.Visible = !IsValue("DisableCountry");
            CompanyRow.Visible = !IsValue("DisableCompany");
            EmailRow.Visible = !IsValue("DisableEmail");
            FirstnameRow.Visible = !IsValue("DisableFirstname");
            LanguageRow.Visible = !IsValue("DisableLanguage");
            LastnameRow.Visible = !IsValue("DisableLastname");
            TitleRow.Visible = !IsValue("DisableTitle");
        }

        #region Web Form Designer generated code
        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Init"/> event.
        /// </summary>
        /// <param name="e">An <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        override protected void OnInit(EventArgs e)
        {
            //
            // CODEGEN: This call is required by the ASP.NET Web Form Designer.
            //
            InitializeComponent();
            base.OnInit(e);
        }

        /// <summary>
        ///		Required method for Designer support - do not modify
        ///		the contents of this method with the code editor.
        /// </summary>
        private void InitializeComponent()
        {
            this.Load += new System.EventHandler(this.Page_Load);
        }
        #endregion
    }
}