#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

using System;
using EPiServer.Personalization;

namespace EPiServer.Templates.Demo.Workroom.Core
{
    /// <summary>
    /// Representation of a workroom membership.
    /// </summary>
    [Serializable]
    public struct MembershipEntry
    {
        private string _name;
        private string _eMail;
        private MembershipLevel _membership;


        /// <summary>
        /// Initializes a new instance of the <see cref="MembershipEntry"/>.
        /// </summary>
        /// <param name="userName">Name of the member.</param>
        /// <param name="membership">The membeship level for the member.</param>
        public MembershipEntry(string userName, MembershipLevel membership)
        {
            _name = userName;
            _membership = membership;
            _eMail = null;
        }

        /// <summary>
        /// Gets or sets the name of the member.
        /// </summary>
        public string Name
        {
            get { return _name; }
            set { _name = value; }
        }

        /// <summary>
        /// Gets or sets en email address of the member
        /// </summary>
        /// <remarks>If no email address has been set it's fetched when requested from profile or membership data.</remarks>
        public string EMail
        {
            get 
            {
                if (_eMail == null) 
                {
                    _eMail = ResolveEmail(Name);
                }
                return _eMail; 
            }
        }

        /// <summary>
        /// Gets or sets the membership level for the member.
        /// </summary>
        public MembershipLevel Membership
        {
            get { return _membership; }
            set { _membership = value; }
        }

        /// <summary>
        /// Returns the localized name of the <see cref="Membership"/>.
        /// </summary>
        /// <value>The name of the membership local.</value>
        public string MembershipLocalName
        {
            get { return Core.Membership.TranslateMembershipLevel(Membership); }
        }

        /// <summary>
        /// Returns a string representation of the entry, containing the user name and the membership level.
        /// </summary>
        public override string ToString()
        {
            return Name + ": " + Membership.ToString();
        }

        private static string ResolveEmail(string userName)
        {
            EPiServerProfile profile = EPiServer.Personalization.EPiServerProfile.Get(userName);
            return (profile != null ? profile.EmailWithMembershipFallback : String.Empty);
        }

        /// <summary>
        /// Indicates whether this instance and a specified object are equal.
        /// </summary>
        /// <param name="obj">Another object to compare to.</param>
        /// <returns>
        /// true if <paramref name="obj"/> and this instance are the same type and represent the same value; otherwise, false.
        /// </returns>
        public override bool Equals(object obj)
        {
            if (!(obj is MembershipEntry))
                return false;
            return String.Equals(Name, ((MembershipEntry)obj).Name);
        }

        /// <summary>
        /// Returns the hash code for this instance.
        /// </summary>
        /// <returns>
        /// A 32-bit signed integer that is the hash code for this instance.
        /// </returns>
        public override int GetHashCode()
        {
            if (Name == null)
                return 0;
            return Name.GetHashCode();
        }

    }
}
