#region Copyright
/*
Copyright (c) 2008 EPiServer AB.  All rights reserved.

This code is released by EPiServer AB under the Source Code File - Specific License Conditions, published August 20, 2007. 
See http://www.episerver.com/Specific_License_Conditions for details. 
*/
#endregion

#region Known Limitations
/*
The template has the following known limitations:

- Usability limitations
  CalendarView selection won't work without javascript.
  
- Performance Limitations
  Using two calendar lists may not be optimal performance wise.

- Architecture limitations
  Calendar item pages has to be visible in menu. 
*/
#endregion

using System;
using System.Text;
using System.Web;
using System.Web.UI;
using System.Web.UI.HtmlControls;
using System.Web.UI.WebControls;

using EPiServer.Core;
using EPiServer.DataAbstraction;
using EPiServer.DataAccess;
using EPiServer.Templates.Demo.Workroom.Core;
using EPiServer.Web.WebControls;

namespace EPiServer.Templates.Demo.Workroom.Pages
{
    public partial class Calendar : WorkroomPageBase
    {
        private static readonly string calendarEventPageTypeName = "[Demo] Workroom Calendar event";
        private static readonly string dateFormat = "dd MMM yyyy";

        /// <summary>
        /// Raises the <see cref="E:System.Web.UI.Control.Load"/> event.
        /// </summary>
        /// <param name="e">The <see cref="T:System.EventArgs"/> object that contains the event data.</param>
        protected override void OnLoad(EventArgs e)
        {
            base.OnLoad(e);

            if (!IsPostBack)
            {
                // Default begin date is todays date.
                CalendarList.BeginDate = DateTime.Now;
                CalendarList.DataBind();

                // MonthlyItems is a hidden calendar list used to calculate what days to highlight in CalenderView.
                // We use the first of current month as begin date but substract 6 days since CalendarView can show up to 6 days of previous month.
                MonthlyItems.BeginDate = new DateTime(CalendarView.TodaysDate.Year, CalendarView.TodaysDate.Month, 1).AddDays(-6);
                // NumberOfDaysToRender is calculated to 6 + 31 + 13 = 50.
                // 6 = CalendarView can show up to 6 days of previous month.
                // 31 = Maximum number of days in a month.
                // 13 = CalendarView can show up to 13 days of next month.
                MonthlyItems.NumberOfDaysToRender = 50;
                MonthlyItems.DataBind();
            }

            if(QueryDistinctMembershipLevel(MembershipLevel.Modify))
            {
                AddButton.Visible = true;
            }

            CalendarView.SelectionChanged += new EventHandler(CalendarView_SelectionChanged);
            CalendarView.DayRender += new DayRenderEventHandler(CalendarView_DayRender);
            CalendarView.VisibleMonthChanged += new MonthChangedEventHandler(CalendarView_VisibleMonthChanged);
        }

        /// <summary>
        /// Gets a display string for the selected period.
        /// </summary>
        /// <returns>A string with start and end date of selected period</returns>
        protected string GetSelectedPeriod()
        {
            StringBuilder returnValue = new StringBuilder();

            if (CalendarList.SelectedDates.Count == 1)
            {
                returnValue.Append(CalendarList.SelectedDates[0].ToString(dateFormat));
            }
            else if (CalendarList.SelectedDates.Count > 1)
            {
                returnValue.Append(CalendarList.SelectedDates[0].ToString(dateFormat));
                returnValue.Append(" &#8722; ");
                returnValue.Append(CalendarList.SelectedDates[CalendarList.SelectedDates.Count - 1].ToString(dateFormat));
            }
            else
            {
                DateTime beginDate = CalendarList.BeginDate;
                returnValue.Append(beginDate.ToString(dateFormat));
                returnValue.Append(" &#8722; ");
                returnValue.Append(beginDate.AddDays(CalendarList.NumberOfDaysToRender).ToString(dateFormat));
            }
            
            return returnValue.ToString();
        }

        /// <summary>
        /// Setups the data for the Calendar list.
        /// </summary>
        protected void SetupData()
        {
            CalendarList.SelectedDates = CalendarView.SelectedDates;
            CalendarList.DataBind();
        }

        #region CalendarView Event Handlers
        
        private void CalendarView_SelectionChanged(object sender, EventArgs e)
        {
            SetupData();
        }

        private void CalendarView_VisibleMonthChanged(object sender, MonthChangedEventArgs e)
        {
            // Set BeginDate for hidden calendar list.
            // Substract 6 days since calendar view can show up to 6 days of previous month.
            MonthlyItems.BeginDate = e.NewDate.AddDays(-6);
        }

        private void CalendarView_DayRender(object sender, DayRenderEventArgs e)
        {
            TableCell cell = e.Cell;
            
            if (MonthlyItems.DateIsActive(e.Day.Date))
            {
                cell.BackColor = System.Drawing.Color.Empty;
                cell.ForeColor = System.Drawing.Color.White;
                cell.CssClass += " highlighted";
            }
        }
        
        #endregion

        #region Button event handlers

        /// <summary>
        /// Creates a new page of the calendar event type and saves it to the database with the values entered by the user.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void SaveButton_Click(object sender, EventArgs e)
        {
            PageData newCalendarEvent = DataFactory.Instance.GetDefaultPageData(CurrentPageLink, calendarEventPageTypeName);

            try
            {
                DateTime startDate = DateTime.Parse(EditEventStartDate.Text);
                DateTime stopDate = DateTime.Parse(EditEventStopDate.Text);

                if (startDate > stopDate)
                {
                    Page.Validators.Add(new StaticValidator(Translate("/workroom/calendar/latestarttime")));
                    return;
                }

                newCalendarEvent.PageName = EditPageName.Text;
                newCalendarEvent["MainBody"] = EditMainBody.Text;
                newCalendarEvent["EventStartDate"] = startDate;
                newCalendarEvent["EventStopDate"] = stopDate;
                newCalendarEvent.StartPublish = DateTime.Now.AddMinutes(-1);

                DataFactory.Instance.Save(newCalendarEvent, SaveAction.Publish);

                EnableViewMode();
            }
            catch (Exception x)
            {
                Page.Validators.Add(new StaticValidator(x.Message));
            }
        }

        /// <summary>
        /// Switches from edit mode to view mode.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void CancelButton_Click(object sender, EventArgs e)
        {
            EnableViewMode();
        }

        /// <summary>
        /// Switches from view mode to edit mode.
        /// </summary>
        /// <param name="sender">The source of the event.</param>
        /// <param name="e">The <see cref="System.EventArgs"/> instance containing the event data.</param>
        protected void AddButton_Click(object sender, EventArgs e)
        {
            EnableEditMode();
        }

        #endregion

        #region Private helper methods

        private void EnableViewMode()
        {
            EditPanel.Visible = false;
            EditPageName.Text = EditMainBody.Text = EditEventStartDate.Text = EditEventStopDate.Text = String.Empty;

            CalendarView.Enabled = true;
            DisplayPanel.Visible = true;
            AddButton.Visible = true;
        }

        private void EnableEditMode()
        {
            CalendarView.Enabled = false;
            DisplayPanel.Visible = false;
            AddButton.Visible = false;

            EditPanel.Visible = true;
        }

        #endregion
    }
}

